/*
 * Created on Feb 28, 2005
 *
 */
package javaMIDISoundServer;

import java.io.*;
import java.net.Socket;
import java.util.StringTokenizer;

/**
 * @author dstulken
 *
 * ServerThread.java - helper class for JaMISS.java.
 * (handles connections once established)
 * part of-
 * JaMISS.java - Java MIDI Sound Server for ISU HCI research
 * (c) 2006 David Stulken, Iowa State University
 *
 */
public class ServerThread extends Thread{
	
	//declare variables
	Socket socketA;
	String clientDataString;
	boolean param_diag=false;
		
	public ServerThread(Socket inputsocket, boolean diagnostic){
		socketA=inputsocket;
		param_diag = diagnostic;
	}
	
	public void run(){
	
	try{
		//create input stream for receiving data from client
		InputStreamReader instream = new InputStreamReader(socketA.getInputStream());
		BufferedReader inputA = new BufferedReader(instream);

		//create output stream for sending data to client... not necessary, but we might as well.
		//  note: set to auto-flush.  Not sure if necessary, but I see no reason why it would hurt.
		PrintWriter outputA = new PrintWriter(socketA.getOutputStream(), true);
	
		//receive data from client
		clientDataString=inputA.readLine();  	//read client data from socket

		//send a direct copy of input string back to client
		outputA.println("Server received command: " + clientDataString);
   
		//done, so close everything (backwards order from creation!)
		outputA.close();	//close PrintWriter 
		socketA.close();	//close socket
	}
	catch(Exception e){
		//print generic error message
		System.out.println("*** Error in server thread! ***");
		System.err.println(e);
	}


	//now that connection is done, we process the data that we read.

	//extract data from input
	StringTokenizer token;
	token = new StringTokenizer(clientDataString, ","); //set up tokens
	String[]clientData = new String[5];
	if (token.hasMoreTokens()){
			clientData[0]=new String(token.nextToken());	//read header
	}
	if (clientData[0].compareTo("JaMISS")!=0){
		//if it is NOT our program that has connected, we must stop.
		System.out.println("*** Error! Invalid data! ***");
		System.out.println("Header should be 'JaMISS'. Received: "+clientData[0]);
		System.out.println(">> String Received: "+clientDataString);
	}
	else
	{
		int tokenCount=1;
		while (token.hasMoreTokens()){
	  		clientData[tokenCount] = new String(token.nextToken()); //read rest of items
	  		tokenCount++; //increment token counter
	  	}
		
		//print one status line regardless of mode
		System.out.println("  >> Command (raw): "+clientDataString);
		
		if (param_diag==true){
			//if in diagnostic mode, print detailed results:
			System.out.println("  >> \tParsed Data:");
			System.out.println("  >> \tHeader (should be 'JaMISS')                   :"+clientData[0]);
			if (clientData[1].compareTo("off")==0){
				System.out.println("  >> \t- optional command found -");
				System.out.println("  >> \t    Note Off (should be 'off')                :"+clientData[1]);
			}
			else if (clientData[1].compareTo("quit")==0){
				System.out.println("  >> \t- optional command found -");
				System.out.println("  >> \t    System Off (should be 'quit')             :"+clientData[1]);
				System.out.println("Note: This does not actually quit while in diagnostic mode.");
			}
			else{
				System.out.println("  >> \t- standard command found -");
				System.out.println("  >> \t    Channel (should be in range0-9 or 11-15)  :"+clientData[1]);
				System.out.println("  >> \t    Note (should be in range 0-127)           :"+clientData[2]);
				System.out.println("  >> \t    Instrument (should be in range 0-127)     :"+clientData[3]);
				System.out.println("  >> \t    Velocity/Volume (should be in range 0-127):"+clientData[4]);				
			}
		} //end diagnostic mode
		else
		{
			//if not in diagnostic mode, update sound!
			if (clientData[1].compareToIgnoreCase("off")==0 || clientData[1].compareToIgnoreCase("quit")==0){
				//optional command found, so send it.
				JaMISS.updateSound(clientData[1], 0, 0, 0, 0);
			}
			else{
				//no optional command, update with values
				try{
					int clientData1=Integer.parseInt(clientData[1]);
					int clientData2=Integer.parseInt(clientData[2]);
					int clientData3=Integer.parseInt(clientData[3]);
					int clientData4=Integer.parseInt(clientData[4]);
					
					JaMISS.updateSound("", clientData1, clientData2, clientData3, clientData4);
				}
				catch (NumberFormatException e){
					System.out.println("  >> Invalid message sent! Please double check your messages!");
					System.out.println("  >> Sound not updated.");
				}
			}
			 
			
		}
	}

	} //end of "run" method

}
